@@ -60,7 +60,9 @@ def print_file_context(
             written += len(to_write)
             output.write(to_write)
         if extra_bytes:
-            output.write(f" [ … plus {extra_bytes} additional byte{['', 's'][extra_bytes > 1]} … ]")
+            output.write(
+                f" [ … plus {extra_bytes} additional byte{['', 's'][extra_bytes > 1]} … ]"
+            )
         output.write("\n")
         if highlight_length < 0 <= highlight_start:
             highlight_length = written - highlight_start
@@ -86,7 +88,9 @@ class ControlFlowDiff:
         if self._diffed:
             return
         self._diffed = True
-        if not self.trace1.has_function(self.func) or not self.trace2.has_function(self.func):
+        if not self.trace1.has_function(self.func) or not self.trace2.has_function(
+            self.func
+        ):
             return
         func1 = self.trace1.get_function(self.func)
         func2 = self.trace2.get_function(self.func)
@@ -152,9 +156,15 @@ class TraceDiff:
         for fname in {
             func.name
             for func in self.trace1.functions
-            if func.name not in {f.name for f in (self.functions_only_in_first | self.functions_only_in_second)}
+            if func.name
+            not in {
+                f.name
+                for f in (self.functions_only_in_first | self.functions_only_in_second)
+            }
         }:
-            yield fname, self.trace1.get_function(fname).taints().diff(self.trace2.get_function(fname).taints())
+            yield fname, self.trace1.get_function(fname).taints().diff(
+                self.trace2.get_function(fname).taints()
+            )
 
     def _diff_functions(self):
         if self._functions_only_in_first is None:
@@ -167,7 +177,9 @@ class TraceDiff:
         if self._bytes_only_in_first is not None:
             return
         # TODO: Instead of looking at what functions touched, just look at the bytes in the canonical mapping!
-        with tqdm(desc="Diffing tainted byte regions", leave=False, unit=" trace", total=2) as t:
+        with tqdm(
+            desc="Diffing tainted byte regions", leave=False, unit=" trace", total=2
+        ) as t:
             for func in tqdm(
                 self.trace1.functions,
                 desc="Trace 1",
@@ -175,7 +187,9 @@ class TraceDiff:
                 leave=False,
             ):
                 for region in func.get_taints().regions():
-                    self._first_intervals[region.source].add(Interval(region.offset, region.offset + region.length))
+                    self._first_intervals[region.source].add(
+                        Interval(region.offset, region.offset + region.length)
+                    )
             for interval in self._first_intervals.values():
                 interval.merge_overlaps()
             t.update(1)
@@ -186,7 +200,9 @@ class TraceDiff:
                 leave=False,
             ):
                 for region in func.get_taints().regions():
-                    self._second_intervals[region.source].add(Interval(region.offset, region.offset + region.length))
+                    self._second_intervals[region.source].add(
+                        Interval(region.offset, region.offset + region.length)
+                    )
             for interval in self._second_intervals.values():
                 interval.merge_overlaps()
             t.update(2)
@@ -201,16 +217,28 @@ class TraceDiff:
                     unit=" intervals",
                     leave=False,
                 ):
-                    self._bytes_only_in_first[source].remove_overlap(interval.begin, interval.end)
-                self._bytes_only_in_second[source] = self._second_intervals[source].copy()
+                    self._bytes_only_in_first[source].remove_overlap(
+                        interval.begin, interval.end
+                    )
+                self._bytes_only_in_second[source] = self._second_intervals[
+                    source
+                ].copy()
                 for interval in tqdm(
                     self._first_intervals[source],
                     desc="Removing Trace 2 Overlap",
                     unit=" intervals",
                     leave=False,
                 ):
-                    self._bytes_only_in_second[source].remove_overlap(interval.begin, interval.end)
-                assert len(self._bytes_only_in_first[source] & self._bytes_only_in_second[source]) == 0
+                    self._bytes_only_in_second[source].remove_overlap(
+                        interval.begin, interval.end
+                    )
+                assert (
+                    len(
+                        self._bytes_only_in_first[source]
+                        & self._bytes_only_in_second[source]
+                    )
+                    == 0
+                )
             for source in self._first_intervals.keys() - self._second_intervals.keys():
                 # sources only in first
                 self._bytes_only_in_first[source] = self._first_intervals[source]
@@ -224,7 +252,11 @@ class TraceDiff:
             self._diff_bytes()
         for source, tree in self._bytes_only_in_first.items():  # type: ignore
             for interval in sorted(tree):
-                yield TaintedRegion(source=source, offset=interval.begin, length=interval.end - interval.begin)
+                yield TaintedRegion(
+                    source=source,
+                    offset=interval.begin,
+                    length=interval.end - interval.begin,
+                )
 
     @property
     def input_chunks_only_in_second(self) -> Iterator[TaintedRegion]:
@@ -232,7 +264,11 @@ class TraceDiff:
             self._diff_bytes()
         for source, tree in self._bytes_only_in_second.items():  # type: ignore
             for interval in sorted(tree):
-                yield TaintedRegion(source=source, offset=interval.begin, length=interval.end - interval.begin)
+                yield TaintedRegion(
+                    source=source,
+                    offset=interval.begin,
+                    length=interval.end - interval.begin,
+                )
 
     @property
     def has_input_chunks_only_in_first(self) -> bool:
@@ -253,8 +289,10 @@ class TraceDiff:
             num_bytes = source.size
             return file_diff(
                 num_bytes,
-                lambda offset: source in self._first_intervals and self._first_intervals[source].overlaps(offset),
-                lambda offset: source in self._second_intervals and self._second_intervals[source].overlaps(offset),
+                lambda offset: source in self._first_intervals
+                and self._first_intervals[source].overlaps(offset),
+                lambda offset: source in self._second_intervals
+                and self._second_intervals[source].overlaps(offset),
             )
 
     def __bool__(self):
@@ -278,22 +316,31 @@ class TraceDiff:
 
         if self.has_input_chunks_only_in_first:
             status.write(
-                "The reference trace touched the following byte regions that were not touched by the diffed " "trace:\n"
+                "The reference trace touched the following byte regions that were not touched by the diffed "
+                "trace:\n"
             )
             for region in self.input_chunks_only_in_first:
                 print_chunk_info((region,))
                 for func in self.trace1.functions:
                     if IntervalTree.from_tuples(
-                        (r.offset, r.offset + r.length) for r in func.taints().regions() if r.source == region.source
+                        (r.offset, r.offset + r.length)
+                        for r in func.taints().regions()
+                        if r.source == region.source
                     ).overlaps(region.offset, region.offset + region.length):
                         # find the control flows that could have caused the diff
                         cfd = ControlFlowDiff(self.trace1, self.trace2, func.name)
                         if cfd:
-                            different_function = cfd.first_function_with_different_control_flow
+                            different_function = (
+                                cfd.first_function_with_different_control_flow
+                            )
                             function_diff = (
                                 self.trace1.get_function(different_function)
                                 .taints()
-                                .diff(self.trace2.get_function(different_function).taints())
+                                .diff(
+                                    self.trace2.get_function(
+                                        different_function
+                                    ).taints()
+                                )
                             )
                             if not bool(function_diff):
                                 continue
@@ -303,7 +350,8 @@ class TraceDiff:
                             )
                             if function_diff.bytes_only_in_first:
                                 status.write(
-                                    "\t\tHere are the bytes that affected control flow only in the reference " "trace:\n"
+                                    "\t\tHere are the bytes that affected control flow only in the reference "
+                                    "trace:\n"
                                 )
                                 print_chunk_info(
                                     function_diff.regions_only_in_first,
@@ -311,7 +359,8 @@ class TraceDiff:
                                 )
                             if function_diff.bytes_only_in_second:
                                 status.write(
-                                    "\t\tHere are the bytes that affected control flow only in the differed " "trace:\n"
+                                    "\t\tHere are the bytes that affected control flow only in the differed "
+                                    "trace:\n"
                                 )
                                 print_chunk_info(
                                     function_diff.regions_only_in_second,
@@ -320,22 +369,31 @@ class TraceDiff:
 
         if self.has_input_chunks_only_in_second:
             status.write(
-                "The diffed trace touched the following byte regions that were not touched by the reference " "trace:\n"
+                "The diffed trace touched the following byte regions that were not touched by the reference "
+                "trace:\n"
             )
             for region in self.input_chunks_only_in_second:
                 print_chunk_info((region,))
                 for func in self.trace2.functions:
                     if IntervalTree.from_tuples(
-                        (r.offset, r.offset + r.length) for r in func.taints().regions() if r.source == region.source
+                        (r.offset, r.offset + r.length)
+                        for r in func.taints().regions()
+                        if r.source == region.source
                     ).overlaps(region.offset, region.offset + region.length):
                         # find the control flows that could have caused the diff
                         cfd = ControlFlowDiff(self.trace1, self.trace2, func.name)
                         if cfd:
-                            different_function = cfd.first_function_with_different_control_flow
+                            different_function = (
+                                cfd.first_function_with_different_control_flow
+                            )
                             function_diff = (
                                 self.trace1.get_function(different_function)
                                 .taints()
-                                .diff(self.trace2.get_function(different_function).taints())
+                                .diff(
+                                    self.trace2.get_function(
+                                        different_function
+                                    ).taints()
+                                )
                             )
                             if not bool(function_diff):
                                 continue
@@ -345,7 +403,8 @@ class TraceDiff:
                             )
                             if function_diff.bytes_only_in_first:
                                 status.write(
-                                    "\t\tHere are the bytes that affected control flow only in the reference " "trace:\n"
+                                    "\t\tHere are the bytes that affected control flow only in the reference "
+                                    "trace:\n"
                                 )
                                 print_chunk_info(
                                     function_diff.regions_only_in_first,
@@ -353,21 +412,29 @@ class TraceDiff:
                                 )
                             if function_diff.bytes_only_in_first:
                                 status.write(
-                                    "\t\tHere are the bytes that affected control flow only in the differed " "trace:\n"
+                                    "\t\tHere are the bytes that affected control flow only in the differed "
+                                    "trace:\n"
                                 )
                                 print_chunk_info(
                                     function_diff.regions_only_in_second,
                                     indent="\t\t\t",
                                 )
 
-        if not self.has_input_chunks_only_in_first and not self.has_input_chunks_only_in_second:
+        if (
+            not self.has_input_chunks_only_in_first
+            and not self.has_input_chunks_only_in_second
+        ):
             status.write("Both traces consumed the exact same input byte regions\n")
 
         for func in self.functions_only_in_first:
-            status.write(f"Function {func!s} was called in the reference trace but not in the diffed trace\n")
+            status.write(
+                f"Function {func!s} was called in the reference trace but not in the diffed trace\n"
+            )
             print_chunk_info(func.taints().regions())
         for func in self.functions_only_in_second:
-            status.write(f"Function {func!s} was called in the diffed trace but not in the reference trace\n")
+            status.write(
+                f"Function {func!s} was called in the diffed trace but not in the reference trace\n"
+            )
             print_chunk_info(func.taints().regions())
         for fname, func in self.functions_in_both:
             if func:
@@ -395,8 +462,12 @@ class TraceDiffCommand(Command):
     help = "compute a diff of two program traces"
 
     def __init_arguments__(self, parser: ArgumentParser):
-        parser.add_argument("trace1", type=str, help="the output database from the first trace")
-        parser.add_argument("trace2", type=str, help="the output database from the second trace")
+        parser.add_argument(
+            "trace1", type=str, help="the output database from the first trace"
+        )
+        parser.add_argument(
+            "trace2", type=str, help="the output database from the second trace"
+        )
         parser.add_argument(
             "--image",
             type=str,
@@ -421,7 +492,9 @@ class TemporalVisualization(Command):
 
     def __init_arguments__(self, parser):
         parser.add_argument("POLYTRACKER_DB", type=str, help="the trace database")
-        parser.add_argument("OUTPUT_GIF_PATH", type=str, help="the path to which to save the animation")
+        parser.add_argument(
+            "OUTPUT_GIF_PATH", type=str, help="the path to which to save the animation"
+        )
 
     def run(self, args):
         from . import PolyTrackerTrace
