@@ -1,11 +1,21 @@
 from abc import ABC, abstractmethod
 from logging import getLogger
-from typing import Generic, Iterable, Iterator, List, Optional, Tuple, Type, TypeVar, Union
+from typing import (
+    Generic,
+    Iterable,
+    Iterator,
+    List,
+    Optional,
+    Tuple,
+    Type,
+    TypeVar,
+    Union,
+)
 
 from intervaltree import Interval, IntervalTree
 from tqdm import tqdm
 
-from .cfg import DAG
+from .graphs import DAG
 from .tracing import BasicBlockEntry, FunctionInvocation, ProgramTrace, TraceEvent
 
 
@@ -31,7 +41,11 @@ class ParseTree(ABC, Generic[V]):
     def to_dag(self) -> DAG["ParseTree[V]"]:
         dag: DAG[ParseTree[V]] = DAG()
         if self.children:
-            dag.add_edges_from((node, child) for node in self.preorder_traversal() for child in node.children)
+            dag.add_edges_from(
+                (node, child)
+                for node in self.preorder_traversal()
+                for child in node.children
+            )
         else:
             dag.add_node(self)
         return dag
@@ -120,7 +134,9 @@ class ImmutableParseTree(Generic[V], ParseTree[V]):
 
     def clone(self: IPT) -> IPT:
         class IPTNode:
-            def __init__(self, node: IPT, parent: Optional["IPTNode"] = None):  # noqa: F821
+            def __init__(
+                self, node: IPT, parent: Optional["IPTNode"] = None  # noqa: F821
+            ):  # noqa: F821
                 self.node: IPT = node
                 self.children: Optional[List[IPT]] = None
                 self.parent: Optional[IPTNode] = parent
@@ -130,10 +146,15 @@ class ImmutableParseTree(Generic[V], ParseTree[V]):
             ipt_node = to_clone[-1]
             if ipt_node.children is None:
                 ipt_node.children = []
-                to_clone.extend(IPTNode(child, ipt_node) for child in reversed(ipt_node.node.children))
+                to_clone.extend(
+                    IPTNode(child, ipt_node)
+                    for child in reversed(ipt_node.node.children)
+                )
             else:
                 to_clone.pop()
-                cloned = self.__class__(value=ipt_node.node.value, children=ipt_node.children)
+                cloned = self.__class__(
+                    value=ipt_node.node.value, children=ipt_node.children
+                )
                 if ipt_node.parent is not None:
                     assert ipt_node.parent.children is not None
                     ipt_node.parent.children.append(cloned)
@@ -243,16 +264,25 @@ def trace_to_tree(
 
     entrypoint_node = node_type(trace.entrypoint)
     root.children.append(entrypoint_node)
-    function_stack: List[Tuple[FunctionInvocation, N]] = [(trace.entrypoint, entrypoint_node)]
+    function_stack: List[Tuple[FunctionInvocation, N]] = [
+        (trace.entrypoint, entrypoint_node)
+    ]
 
     with tqdm(
-        unit=" functions", leave=False, desc="extracting a parse tree", total=trace.num_function_calls_that_touched_taint()
+        unit=" functions",
+        leave=False,
+        desc="extracting a parse tree",
+        total=trace.num_function_calls_that_touched_taint(),
     ) as t:
         while function_stack:
             function, node = function_stack.pop()
             t.update(1)
             for bb in tqdm(
-                function.basic_blocks(), unit=" basic blocks", leave=False, desc=function.function.demangled_name, delay=1.0
+                function.basic_blocks(),
+                unit=" basic blocks",
+                leave=False,
+                desc=function.function.demangled_name,
+                delay=1.0,
             ):
                 child_node = node_type(bb)
                 node.children.append(child_node)
@@ -262,7 +292,9 @@ def trace_to_tree(
                 func = bb.called_function
                 if func is not None:
                     if not func.touched_taint:
-                        log.debug(f"skipping call to {func.function.demangled_name} because it did not touch taint")
+                        log.debug(
+                            f"skipping call to {func.function.demangled_name} because it did not touch taint"
+                        )
                         continue
                     child_node = node_type(func)
                     node.children.append(child_node)
@@ -275,7 +307,9 @@ G = TypeVar("G", bound="NonGeneralizedParseTree")
 
 
 class NonGeneralizedParseTree(MutableParseTree[Union[Start, TraceEvent, Terminal]]):
-    def __init__(self: G, value: Union[Start, TraceEvent, Terminal], children: Iterable[G] = ()):
+    def __init__(
+        self: G, value: Union[Start, TraceEvent, Terminal], children: Iterable[G] = ()
+    ):
         super().__init__(value=value, children=children)
         self.consumed: List[Tuple[int, int]]
         if isinstance(value, BasicBlockEntry):
@@ -305,18 +339,37 @@ class NonGeneralizedParseTree(MutableParseTree[Union[Start, TraceEvent, Terminal
     def matches(self) -> bytes:
         return b"".join(terminal.terminal for terminal in self.terminals())
 
-    def verify_bounds(self, check_overlap=True, check_coverage=True, check_missing_children=True):
+    def verify_bounds(
+        self, check_overlap=True, check_coverage=True, check_missing_children=True
+    ):
         covered_input_bytes = IntervalTree()
         for child in self.children:
-            if check_overlap and covered_input_bytes.overlaps(child.begin_offset, child.end_offset):
-                overlap = ", ".join([interval.data for interval in covered_input_bytes[child.begin_offset: child.end_offset]])
-                raise ValueError(f"Child node {child.value!s} of {self.value!s} overlaps with these siblings: " f"{overlap!r}")
+            if check_overlap and covered_input_bytes.overlaps(
+                child.begin_offset, child.end_offset
+            ):
+                overlap = ", ".join(
+                    [
+                        interval.data
+                        for interval in covered_input_bytes[
+                            child.begin_offset : child.end_offset
+                        ]
+                    ]
+                )
+                raise ValueError(
+                    f"Child node {child.value!s} of {self.value!s} overlaps with these siblings: "
+                    f"{overlap!r}"
+                )
             if child.end_offset > child.begin_offset:
-                covered_input_bytes.addi(child.begin_offset, child.end_offset, str(child.value))
+                covered_input_bytes.addi(
+                    child.begin_offset, child.end_offset, str(child.value)
+                )
         if (
             check_coverage
             and not self.is_leaf()
-            and (covered_input_bytes.begin() != self.begin_offset or covered_input_bytes.end() != self.end_offset)
+            and (
+                covered_input_bytes.begin() != self.begin_offset
+                or covered_input_bytes.end() != self.end_offset
+            )
         ):
             raise ValueError(
                 f"Node {self.value!s} was expected to have bounds ({self.begin_offset}, "
@@ -326,9 +379,14 @@ class NonGeneralizedParseTree(MutableParseTree[Union[Start, TraceEvent, Terminal
         if check_missing_children and not self.is_leaf():
             covered_input_bytes.merge_overlaps(strict=False)
             if len(covered_input_bytes) > 1:
-                missing = IntervalTree.from_tuples([(self.begin_offset, self.end_offset)]) - covered_input_bytes
+                missing = (
+                    IntervalTree.from_tuples([(self.begin_offset, self.end_offset)])
+                    - covered_input_bytes
+                )
                 missing_str = ", ".join(f"[{i.begin}:{i.end}]" for i in missing)
-                raise ValueError(f"Node {self.value!s} is missing children that cover these byte ranges: {missing_str}")
+                raise ValueError(
+                    f"Node {self.value!s} is missing children that cover these byte ranges: {missing_str}"
+                )
 
     def verify(self, string: bytes):
         offset: int = 0
@@ -352,10 +410,18 @@ class NonGeneralizedParseTree(MutableParseTree[Union[Start, TraceEvent, Terminal
             offset += terminal_length
 
     def simplify(self):
-        for node in tqdm(self.postorder_traversal(), leave=False, desc="simplifying parse tree", unit=" nodes"):
+        for node in tqdm(
+            self.postorder_traversal(),
+            leave=False,
+            desc="simplifying parse tree",
+            unit=" nodes",
+        ):
             if len(node.children) == 1:
                 child = node.children[0]
-                if isinstance(child.value, BasicBlockEntry) and len(child.children) == 1:
+                if (
+                    isinstance(child.value, BasicBlockEntry)
+                    and len(child.children) == 1
+                ):
                     node.children = [child.children[0]]
 
     def _winners(self, to_compare: "NonGeneralizedParseTree") -> Optional[List[int]]:
@@ -384,7 +450,10 @@ class NonGeneralizedParseTree(MutableParseTree[Union[Start, TraceEvent, Terminal
                 their_last_used.append(last_used)
             else:
                 their_last_used.append(-1)
-        return [our_last - their_last for our_last, their_last in zip(our_last_used, their_last_used)]
+        return [
+            our_last - their_last
+            for our_last, their_last in zip(our_last_used, their_last_used)
+        ]
 
     def best_partition(self, right_sibling: "NonGeneralizedParseTree") -> Optional[int]:
         winners = self._winners(right_sibling)
@@ -408,7 +477,9 @@ class NonGeneralizedParseTree(MutableParseTree[Union[Start, TraceEvent, Terminal
     def best_subset(self, parent: "NonGeneralizedParseTree") -> Tuple[int, int]:
         winners = self._winners(parent)
         if winners is None:
-            raise ValueError("The child does not overlap with its parent! This should never happen.")
+            raise ValueError(
+                "The child does not overlap with its parent! This should never happen."
+            )
         # TODO: See if we can improve this algorithm
         left_offset = 0
         right_offset = len(winners)
@@ -421,16 +492,22 @@ class NonGeneralizedParseTree(MutableParseTree[Union[Start, TraceEvent, Terminal
     def deconflict_sibling(self, right_sibling: "NonGeneralizedParseTree"):
         best_point = self.best_partition(right_sibling)
         if best_point is not None:
-            self.intervals.chop(right_sibling.begin_offset + best_point, self.end_offset)
+            self.intervals.chop(
+                right_sibling.begin_offset + best_point, self.end_offset
+            )
             right_sibling.intervals.chop(0, right_sibling.begin_offset + best_point)
 
     def deconflict_parent(self, parent: "NonGeneralizedParseTree"):
         left_offset, right_offset = self.best_subset(parent)
-        self.intervals.chop(self.begin_offset + left_offset, self.end_offset - right_offset)
+        self.intervals.chop(
+            self.begin_offset + left_offset, self.end_offset - right_offset
+        )
 
     def bottom_up_pass(self):
         # first, remove any children that do not produce a terminal
-        self.children = [child for child in self.children if child.begin_offset < child.end_offset]
+        self.children = [
+            child for child in self.children if child.begin_offset < child.end_offset
+        ]
         # ensure that none of our children's intervals overlap
         for child, right_sibling in zip(self.children, self.children[1:]):
             child.deconflict_sibling(right_sibling)
@@ -440,7 +517,9 @@ class NonGeneralizedParseTree(MutableParseTree[Union[Start, TraceEvent, Terminal
         self.intervals.split_overlaps()
         self.intervals.merge_overlaps(data_reducer=max)
         if __debug__:
-            self.verify_bounds(check_overlap=True, check_coverage=False, check_missing_children=False)
+            self.verify_bounds(
+                check_overlap=True, check_coverage=False, check_missing_children=False
+            )
 
     def top_down_pass(self):
         self._begin = self.begin_offset
@@ -471,7 +550,9 @@ def trace_to_non_generalized_tree(trace: ProgramTrace) -> NonGeneralizedParseTre
 
     inputs = list(trace.inputs)
     if len(inputs) != 1:
-        raise ValueError(f"Trace {trace!r} must have exactly one input; found {len(inputs)}")
+        raise ValueError(
+            f"Trace {trace!r} must have exactly one input; found {len(inputs)}"
+        )
     inputstr = inputs[0].content
 
     for node in tqdm(
@@ -482,11 +563,15 @@ def trace_to_non_generalized_tree(trace: ProgramTrace) -> NonGeneralizedParseTre
         unit=" nodes",
     ):
         if isinstance(node.value, Start):
-            node.intervals[0: len(inputstr)] = 0
+            node.intervals[0 : len(inputstr)] = 0
         node.bottom_up_pass()
 
     for node in tqdm(
-        tree.preorder_traversal(), total=tree.descendants + 1, leave=False, desc=" finalizing parse tree ranges", unit=" nodes"
+        tree.preorder_traversal(),
+        total=tree.descendants + 1,
+        leave=False,
+        desc=" finalizing parse tree ranges",
+        unit=" nodes",
     ):
         if isinstance(node.value, Terminal):
             continue
@@ -500,7 +585,9 @@ def trace_to_non_generalized_tree(trace: ProgramTrace) -> NonGeneralizedParseTre
             if child.begin_offset >= child.end_offset:
                 continue
             if last_end < child.begin_offset:
-                terminal = NonGeneralizedParseTree(Terminal(inputstr[last_end: child.begin_offset]))
+                terminal = NonGeneralizedParseTree(
+                    Terminal(inputstr[last_end : child.begin_offset])
+                )
                 terminal.intervals.addi(last_end, child.begin_offset)
                 new_children.append(terminal)
             new_children.append(child)
