@@ -15,16 +15,25 @@ from polytracker import (
 from .data import *
 
 
+@pytest.mark.skip(reason="taint_dag does not support traces yet")
 @pytest.mark.program_trace("test_mmap.c")
 def test_source_mmap(program_trace: ProgramTrace):
-    assert any(byte_offset.offset == 0 for byte_offset in program_trace.get_function("main").taints())
+    assert any(
+        byte_offset.offset == 0
+        for byte_offset in program_trace.get_function("main").taints()
+    )
 
 
+@pytest.mark.skip(reason="taint_dag does not support traces yet")
 @pytest.mark.program_trace("test_open.c")
 def test_source_open(program_trace: ProgramTrace):
-    assert any(byte_offset.offset == 0 for byte_offset in program_trace.get_function("main").taints())
+    assert any(
+        byte_offset.offset == 0
+        for byte_offset in program_trace.get_function("main").taints()
+    )
 
 
+@pytest.mark.skip(reason="taint_dag does not support traces yet")
 @pytest.mark.program_trace("test_control_flow.c")
 def test_control_flow(program_trace: ProgramTrace):
     # make sure the trace contains all of the functions:
@@ -76,11 +85,16 @@ def test_control_flow(program_trace: ProgramTrace):
     # so for now just ignore main
 
 
+@pytest.mark.skip(reason="taint_dag does not support traces yet")
 @pytest.mark.program_trace("test_open.c")
 def test_source_open_full_validate_schema(program_trace: ProgramTrace):
-    assert any(byte_offset.offset == 0 for byte_offset in program_trace.get_function("main").taints())
+    assert any(
+        byte_offset.offset == 0
+        for byte_offset in program_trace.get_function("main").taints()
+    )
 
 
+@pytest.mark.skip(reason="taint_dag does not support traces yet")
 @pytest.mark.program_trace("test_memcpy.c")
 def test_memcpy_propagate(program_trace: ProgramTrace):
     func = program_trace.get_function("touch_copied_byte")
@@ -89,6 +103,7 @@ def test_memcpy_propagate(program_trace: ProgramTrace):
     assert next(iter(taints)).offset == 0
 
 
+@pytest.mark.skip(reason="taint_dag does not support traces yet")
 @pytest.mark.program_trace("test_taint_log.c")
 def test_taint_log(program_trace: ProgramTrace):
     taints = program_trace.get_function("main").taints()
@@ -96,7 +111,10 @@ def test_taint_log(program_trace: ProgramTrace):
         assert any(i == offset.offset for offset in taints)
 
 
-@pytest.mark.program_trace("test_taint_log.c", config_path=CONFIG_DIR / "new_range.json")
+@pytest.mark.skip(reason="taint_dag does not support traces yet")
+@pytest.mark.program_trace(
+    "test_taint_log.c", config_path=CONFIG_DIR / "new_range.json"
+)
 def test_config_files(program_trace: ProgramTrace):
     # the new_range.json config changes the polystart/polyend to
     # POLYSTART: 1, POLYEND: 3
@@ -107,18 +125,21 @@ def test_config_files(program_trace: ProgramTrace):
         assert all(i != offset.offset for offset in taints)
 
 
+@pytest.mark.skip(reason="taint_dag does not support traces yet")
 @pytest.mark.program_trace("test_fopen.c")
 def test_source_fopen(program_trace: ProgramTrace):
     taints = program_trace.get_function("main").taints()
     assert any(offset.offset == 0 for offset in taints)
 
 
+@pytest.mark.skip(reason="taint_dag does not support traces yet")
 @pytest.mark.program_trace("test_ifstream.cpp")
 def test_source_ifstream(program_trace: ProgramTrace):
     taints = program_trace.get_function("main").taints()
     assert any(offset.offset == 0 for offset in taints)
 
 
+@pytest.mark.skip(reason="taint_dag does not support traces yet")
 @pytest.mark.program_trace("test_object_propagation.cpp")
 def test_cxx_object_propagation(program_trace: ProgramTrace):
     for func in program_trace.functions:
@@ -127,11 +148,16 @@ def test_cxx_object_propagation(program_trace: ProgramTrace):
 
 
 # TODO Compute DFG and query if we touch vector in libcxx from object
+@pytest.mark.skip(reason="taint_dag does not support traces yet")
 @pytest.mark.program_trace("test_vector.cpp")
 def test_cxx_vector(program_trace: ProgramTrace):
-    assert any(byte_offset.offset == 0 for byte_offset in program_trace.get_function("main").taints())
+    assert any(
+        byte_offset.offset == 0
+        for byte_offset in program_trace.get_function("main").taints()
+    )
 
 
+@pytest.mark.skip(reason="taint_dag does not support traces yet")
 @pytest.mark.program_trace("test_fgetc.c", input="ABCDEFGH")
 def test_fgetc(program_trace: ProgramTrace):
     for _ in program_trace:
@@ -147,14 +173,16 @@ def test_fgetc(program_trace: ProgramTrace):
     for i, called in enumerate(called_by_main):
         regions = list(called.taints().regions())
         assert len(regions) == 1
-        assert regions[0].value == b"ABCDEFGH"[i: i + 1]
+        assert regions[0].value == b"ABCDEFGH"[i : i + 1]
 
+
+@pytest.mark.skip(reason="taint_dag does not support traces yet")
 @pytest.mark.program_trace("test_cxx_global_object.cpp", taint_all=True)
 def test_cxx_global_object(program_trace: ProgramTrace):
     r1 = program_trace.get_function("_ZN12GlobalObject10read_firstEv")
     assert r1.demangled_name == "GlobalObject::read_first()"
     taints = list(r1.taints().regions())
-    assert len(taints) ==1
+    assert len(taints) == 1
     assert taints[0].offset == 0
     assert taints[0].length == 1
 
@@ -172,10 +200,16 @@ def test_cxx_global_object(program_trace: ProgramTrace):
     assert taints[0].offset == 1
     assert taints[0].length == 1
 
+
 @pytest.mark.program_trace("test_simple_union.cpp", input="ABCDEFGH\n11235878\n")
 def test_taint_forest(program_trace: ProgramTrace):
     had_taint_union = False
-    for taint_node in tqdm(program_trace.taint_forest.nodes(), leave=False, desc="validating", unit=" taint nodes"):
+    for taint_node in tqdm(
+        program_trace.taint_forest.nodes(),
+        leave=False,
+        desc="validating",
+        unit=" taint nodes",
+    ):
         if taint_node.is_canonical():
             assert taint_node.parent_one is None
             assert taint_node.parent_two is None
